@@ -62,20 +62,27 @@ for (i in 2:nvar ) {
 cl <- match.call()
 p <- dim(x)[1]
 len <- p
+
+
 if (dim(x)[2]!= p) {
-x1 <- as.character(x[,1])
-x2 <- as.character(x[,2])
-
-
+x1 <- x[,1]
+x2 <- x[,2]
+if(is.factor(x1) ) {  #this gets around a problem of tabling numbers as characters (bad idea) vs. tabling characters (good idea)
+  x1 <- as.character(x[,1])
+ x2 <- as.character(x[,2])} else {
+  x1 <- x[,1]
+  x2 <- x[,2]}
 labels <- levels(as.factor(cbind(x1,x2)))
-len <- length(labels)
-x <- matrix(0,ncol=len,nrow=len)
-colnames(x) <- rownames(x) <- labels
-x1f <- factor(x1,levels=labels)
-x2f <- factor(x2,levels=labels)
-x <- table(x1f,x2f)
-#for (item in 1:p) {x[x1[item],x2[item]] <- x[x1[item],x2[item]] +1}
-}
+ len <- length(labels)
+ x <- matrix(0,ncol=len,nrow=len)
+ colnames(x) <- rownames(x) <- labels
+ x1f <- factor(x1,levels=labels)
+ x2f <- factor(x2,levels=labels)
+ x <- table(x1f,x2f)
+ 
+
+# #for (item in 1:p) {x[x1[item],x2[item]] <- x[x1[item],x2[item]] +1}
+ }
 
 x <- as.matrix(x)
 tot <- sum(x)
