@@ -24,20 +24,36 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,s
      				 sds <- sqrt(diag(r))    #convert covariance matrices to correlation matrices
                      if(!covar) r <- r/(sds %o% sds)  }  #added June 9, 2008
     if (!residuals) { result <- list(values=c(rep(0,n)),rotation=rotate,n.obs=n.obs,communality=c(rep(0,n)),loadings=matrix(rep(0,n*n),ncol=n),fit=0,fit.off=0)} else { result <- list(values=c(rep(0,n)),rotation=rotate,n.obs=n.obs,communality=c(rep(0,n)),loadings=matrix(rep(0,n*n),ncol=n),residual=matrix(rep(0,n*n),ncol=n),fit=0,fit.off=0)}
+    #added 24/4/15 to stop with bad data and give more meaningful help
+    if(any(is.na(r))) {
+       bad <- TRUE
+      tempr <-r
+      wcl <-NULL
+     while(bad) {
+     	wc <- table(which(is.na(tempr), arr.ind=TRUE))  #find the correlations that are NA
+    	wcl <- c(wcl,as.numeric(names(which(wc==max(wc)))))
+    	tempr <- r[-wcl,-wcl]
+    	if(any(is.na(tempr))) {bad <- TRUE} else {bad <- FALSE}
+         }
+
+     	cat('\nLikely variables with missing values are ',colnames(r)[wcl],' \n')
+      	 stop("I am sorry: missing values (NAs) in the correlation matrix do not allow me to continue.\nPlease drop those variables and try again." )
+       }
+    
     eigens <- eigen(r)    #call the eigen value decomposition routine
     result$values <- eigens$values
     eigens$values[ eigens$values < .Machine$double.eps] <-  .Machine$double.eps  #added May 14, 2009 to fix case of singular matrices
     loadings <- eigens$vectors %*% sqrt(diag(eigens$values))
    
-   if(nfactors >0) {loadings <- loadings[,1:nfactors]} else {nfactors <- n}
-   if (nfactors>1) {communalities <- rowSums(loadings^2)} else {communalities <- loadings^2 }
+   if(nfactors > 0) {loadings <- loadings[,1:nfactors]} else {nfactors <- n}
+   if (nfactors > 1) {communalities <- rowSums(loadings^2)} else {communalities <- loadings^2 }
    uniquenesses <- diag(r) - communalities # 2011.12.21 uniqueness is now found if covar is true
     names(communalities) <- colnames(r)    # 2009.02.10 Make sure this is a named vector -- correction by Gumundur Arnkelsson
    	
    
     	 
      #added January 19, 2009 to flip based upon colSums of loadings
-    if (nfactors >1) {sign.tot <- vector(mode="numeric",length=nfactors)
+    if (nfactors > 1) {sign.tot <- vector(mode="numeric",length=nfactors)
                  sign.tot <- sign(colSums(loadings))
                  sign.tot[sign.tot==0] <- 1
                  loadings <- loadings %*% diag(sign.tot)
@@ -47,41 +63,80 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,s
     	
     colnames(loadings) <- paste("PC",1:nfactors,sep='')
     rownames(loadings) <- rownames(r)
+  Phi <- NULL   
+ if(rotate != "none") {if (nfactors > 1) {
+if (rotate=="varimax" |rotate=="Varimax" | rotate=="quartimax" | rotate =="bentlerT" | rotate =="geominT" | rotate =="targetT" | rotate =="bifactor"   | rotate =="TargetT"|
+                       rotate =="equamax"| rotate =="varimin"|rotate =="specialT" | rotate =="Promax"  | rotate =="promax"| rotate =="cluster" |rotate == "biquartimin" |rotate == "TargetQ"  |rotate =="specialQ" ) {
 
-     Phi <- NULL
-      if(rotate != "none") {if (nfactors > 1) {
-  
-   	if (rotate=="varimax" |rotate=="Varimax" | rotate=="quartimax" | rotate =="bentlerT" | rotate =="geominT" |rotate =="bifactor"  |rotate =="varimin" |rotate =="targetT" |rotate =="equamax") { 
-   	if (!requireNamespace('GPArotation')) {stop("I am sorry, to do these rotations requires the GPArotation package to be installed")}
+switch(rotate,  #The orthogonal cases  for GPArotation + ones developed for psych
+  varimax = {rotated <- stats::varimax(loadings)  #varimax is from stats, the others are from GPArotation 
+   			         loadings <- rotated$loadings},
+   Varimax = {if (!requireNamespace('GPArotation')) {stop("I am sorry, to do this rotation requires the GPArotation package to be installed")}
    	       #varimax is from the stats package, Varimax is from GPArotations
-   	       colnames(loadings) <- paste("RC",1:nfactors,sep='')  #to remind us that we are rotating the components 
-   			rotated <- do.call(rotate,list(loadings,...))
-   			loadings <- rotated$loadings
+   			#rotated <- do.call(rotate,list(loadings,...))
+   			#rotated <- do.call(getFromNamespace(rotate,'GPArotation'),list(loadings,...))
+   			rotated <- GPArotation::Varimax(loadings)
+   			loadings <- rotated$loadings} ,
+   	quartimax = {if (!requireNamespace('GPArotation')) {stop("I am sorry, to do this rotation requires the GPArotation package to be installed")}
+   	      
+   			#rotated <- do.call(rotate,list(loadings))
+   			rotated <- GPArotation::quartimax(loadings)
+   			loadings <- rotated$loadings} ,
+   	bentlerT =  {if (!requireNamespace('GPArotation')) {stop("I am sorry, to do this rotation requires the GPArotation package to be installed")}
+   	       
+   			#rotated <- do.call(rotate,list(loadings,...))
+   			rotated <- GPArotation::bentlerT(loadings)
+   			loadings <- rotated$loadings} ,
+   	geominT	= {if (!requireNamespace('GPArotation')) {stop("I am sorry, to do this rotation requires the GPArotation package to be installed")}
+   	      
+   			#rotated <- do.call(rotate,list(loadings,...))
+   			rotated <- GPArotation::geominT(loadings)
+   			loadings <- rotated$loadings} ,
+   	targetT = {if (!requireNamespace('GPArotation')) {stop("I am sorry, to do this rotation requires the GPArotation package to be installed")}
+   	     
+   			#rotated <- do.call(rotate,list(loadings,...))
+   			rotated <- GPArotation::targetT(loadings)
+   			loadings <- rotated$loadings} ,
    			
-   			 Phi <- NULL} else { 
-   			
-     			if ((rotate=="promax")|(rotate=="Promax") ) {pro <- Promax(loadings)
-     			                loadings <- pro$loadings
-     			                Phi <- pro$Phi} else {
-     			if (rotate == "cluster") {loadings <- varimax(loadings)$loadings           			
-								pro <- target.rot(loadings,...)
+   	 bifactor = {rotated <- bifactor(loadings)$loadings},
+   	 TargetT =  {rotated <- TargetT(loadings,...)$loadings},
+   	equamax =  {rotated <- equamax(loadings)$loadings}, 
+   	varimin = {rotated <- varimin(loadings)$loadings},
+   	specialT =  {rotated <- specialT(loadings)$loadings}, 
+   	Promax =   {pro <- Promax(loadings)
+     			 loadings <- pro$loadings
+     			  Phi <- pro$Phi },
+     promax =   {pro <- Promax(loadings)
+     			 loadings <- pro$loadings
+     			  Phi <- pro$Phi },	
+     cluster = 	 {loadings <- varimax(loadings)$loadings           			
+								pro <- target.rot(loadings)
      			              	loadings <- pro$loadings
-     			                Phi <- pro$Phi} else {
-     			                     
-     			if (rotate =="oblimin"| rotate=="quartimin" | rotate== "simplimax" | rotate =="geominQ"  | rotate =="bentlerQ" |rotate == "biquartimin" |rotate == "targetQ" |rotate == "TargetQ" ) {
+     			                Phi <- pro$Phi},
+     biquartimin =    {ob <- biquartimin(loadings,)
+                    loadings <- ob$loadings
+     				 Phi <- ob$Phi}, 
+     TargetQ  =  {ob <- TargetQ(loadings,...)
+                    loadings <- ob$loadings
+     				 Phi <- ob$Phi}, 
+     specialQ = {ob <- specialQ(loadings,...)
+                    loadings <- ob$loadings
+     				 Phi <- ob$Phi})
+     } else {
+     #The following oblique cases all use GPArotation			                
+     if (rotate =="oblimin"| rotate=="quartimin" | rotate== "simplimax" | rotate =="geominQ"  | rotate =="bentlerQ"  |rotate == "targetQ"  ) {
      				if (!requireNamespace('GPArotation')) {warning("I am sorry, to do these rotations requires the GPArotation package to be installed")
-     				Phi <- NULL} else {
-     				 colnames(loadings) <- paste("TC",1:nfactors,sep='')  #to remind us that we are Transforming the components 
-     				  ob  <- try(do.call(rotate,list(loadings,...) ))
-     				          if(class(ob)== as.character("try-error"))  {warning("The requested transformaton failed, Promax was used instead as an oblique transformation")
-     				                  ob <- Promax(loadings)}
+     				    Phi <- NULL} else { 
+     				      
+     				             ob <- try(do.call(getFromNamespace(rotate,'GPArotation'),list(loadings,...)))
+     				               if(class(ob)== as.character("try-error"))  {warning("The requested transformaton failed, Promax was used instead as an oblique transformation")
+     				               ob <- Promax(loadings)}
      				                 
      				loadings <- ob$loadings
      				 Phi <- ob$Phi}
-     		                              } else {message("Specified rotation not found, rotate='none' used")} #added 30/11/14 
-     	               }}}
-    
-     }}
+     		                             } else {message("Specified rotation not found, rotate='none' used")}
+     	 } }}
+     	 		
    #just in case the rotation changes the order of the components, sort them by size of eigen value
    if(nfactors >1) {
     ev.rotated <- diag(t(loadings) %*% loadings)
@@ -109,10 +164,14 @@ function(r,nfactors=1,residuals=FALSE,rotate="varimax",n.obs = NA, covar=FALSE,s
     result$Call <- cl
     result$communality <- communalities
     result$uniquenesses <- uniquenesses
+    result$complexity <- stats$complexity
     #result$stats <- stats
+    result$chi <- stats$chi
+    result$EPVAL <- stats$EPVAL
     result$R2 <- stats$R2
     result$objective <- stats$objective
     result$residual <- stats$residual
+    result$rms <- stats$rms
     result$fit <-  stats$fit
     result$fit.off <-  stats$fit.off
     result$factors <-  stats$factors
