@@ -101,4 +101,13 @@ function(header=FALSE,widths=rep(1,10),...) {  #
     else {
    if (header) read.clipboard<- read.fwf(pipe("pbpaste"),header=TRUE,widths=widths,...)
    else read.clipboard<- read.fwf(pipe("pbpaste"),widths=widths,...)}
-   }
\ No newline at end of file
+   }
+   
+#added May, 2014 to read from https files
+"read.https" <-
+function(filename,header=TRUE) {
+temp <- tempfile()   #create a temporary file
+download.file(filename,destfile=temp,method="curl") #copy the https file to temp
+result <- read.table(temp,header=header) #now, do the normal read.table command
+unlink(temp) #get rid of the temporary file
+return(result)}  #give us the result
\ No newline at end of file
