@@ -1,7 +1,7 @@
 #November 30, 2013
 #parts adapted from combn
 "splitHalf"<- 
-function(r,raw=FALSE,brute=FALSE,n.sample=10000,covar=FALSE) {
+function(r,raw=FALSE,brute=FALSE,n.sample=10000,covar=FALSE,check.keys=TRUE,key=NULL) {
 cl <- match.call()
 split <- function(o,n) {
 A <- B <-  rep(0,n)
@@ -14,7 +14,7 @@ Rab <- R[1,2]/sqrt(R[1,1]*R[2,2])
 rab <- 4*R[1,2]/sum(R)
 result <- list(rab=rab,AB=AB)}
 
-
+keys <- key   
 maxrb <- -9999
 minrb <- 2
 n <- ncol(r)
@@ -22,6 +22,26 @@ n2 <- trunc(n/2)
 n.obs <- nrow(r)
 if(n.obs > n) { r <- cov(r,use="pairwise")}
  if(!covar) r <- cov2cor(r) 
+ 
+if(check.keys && is.null(keys)) {
+            p1 <- principal(r)
+            if(any(p1$loadings < 0)) warning("Some items were negatively correlated with total scale and were automatically reversed.")
+            keys <- 1- 2* (p1$loadings < 0 ) 
+            }  #keys is now a vector of 1s and -1s
+            
+         if(is.null(keys)) {keys <- rep(1,n)} else {  
+         			keys<- as.vector(keys) 
+         			if(length(keys) < n) {temp <- keys  #this is the option of keying just the reversals
+         			                         keys <- rep(1,n)
+         			                         names(keys) <- colnames(r)
+         			                         keys[temp] <- -1}
+         			                   } 
+        			 key.d <- diag(keys)
+                     r <- key.d %*% r %*% key.d
+                     signkey <- strtrim(keys,1)
+            		 signkey[signkey=="1"] <- ""
+                     colnames(r) <- paste(colnames(r),signkey,sep="")
+
 e <- 0
 m <- n2
 h <- m
@@ -34,7 +54,9 @@ lambda6 <- (sumr - tr(r) + tsmc)/sumr
 sumr <- 0 
 x <- seq_len(n)
 a <- seq_len(m)
-count <- as.integer(round(choose(n, m)))/2
+#count <- as.integer(round(choose(n, m)))/2 #doesn't work for very large values of n.
+count <- round(choose(n, m))/2
+
 if(brute || ((count <= n.sample) && !raw)) {    #brute force -- try all combinations
 brute <- TRUE 
  if(raw) result <- rep(NA,count)  #keep the results if raw
@@ -90,6 +112,11 @@ for (i in 1:n.sample) {
  #now 
 if(brute) {meanr <- sumr/count } else {meanr <- sumr/n.sample }
 meansp <- 2 * meanr/(1+meanr)
+
+kd <- diag(key.d)    #reverse them so we can use the keys
+maxAB = maxAB * kd
+minAB = minAB * kd
+
 if(raw) {results <- list(maxrb=maxrb,minrb=minrb,maxAB=maxAB,minAB=minAB,meanr=meanr,alpha=alpha,lambda6=lambda6,raw = result,Call = cl)
 } else {results <- list(maxrb=maxrb,minrb=minrb,maxAB=maxAB,minAB=minAB,meanr=meanr,alpha=alpha,lambda6=lambda6,Call=cl)}
 class(results) <- c("psych","split")
